/*global $, console, app, iScroll, Handlebars, FacebookInAppBrowser, openFB */

var EventView = function (store, event, increaseViewCount) {
  // Application Constructor
  "use strict";
  var self = this;
  this.initialize = function () {
    // Define a div wrapper for the view. The div wrapper is used to attach events.
    this.el = $('<div/>');
    if (event.count == 0) {
      this.el.on('click', '#getFreeStuff', this.getFreeStuff);
    }
  };

  this.render = function () {
    this.el.html(EventView.template(event));
    this.showRewards();
    this.updateViewCount();
    return this;
  };
  
  // Updates the amount of times this event has been viewed.
  this.updateViewCount = function () {
    if (increaseViewCount === false) {
      return;
    }
    console.log('view increased for event: ' + event.id);
    app.executeJsonCall('http://themountaingoat.co.za/webservices/viewIncrementer.php?id=' + event.id, function () { });
  };
  
  // Renders the reward.
  this.showRewards = function () {
    // Check store first. If there is nothing in there, then fetch from the web.
    store.getRewardListByEventId(event.eventDateID, function (storedRewards) {
      if (storedRewards.length > 0) {
        self.populateRewardTemplate(storedRewards);
      } else {
        app.executeJsonCall('http://themountaingoat.co.za/webservices/listRewards.php?id=' + event.eventDateID, function (rewards) {
          store.storeRewardList(rewards, event.eventDateID);
          self.populateRewardTemplate(rewards);
        });
      }
    });
  };
  
  this.populateRewardTemplate = function (rewards) {
    console.log(rewards);
    if (rewards.length >= 1) {
      $('#reward').html(EventView.rewardLiTemplate(rewards[0]));
    } else {
      $('#reward').html(EventView.noItemTemplate({ entity: "rewards" }));
    }
  };
  
  this.getFreeStuff = function () {
    var successFn = function () {
      // Updates the amount of times this event has been shared.
      app.executeJsonCall('http://themountaingoat.co.za/webservices/shareIncrementer.php?id=' + event.id, function () {
        console.log('Share increased for event: ' + event.id);
      });
      
//      app.executeJsonCall('http://themountaingoat.co.za/webservices/createToken.php' +
//                          '?id=' + sessionStorage.getItem('fb_id') +
      app.executeJsonCall('http://themountaingoat.co.za/webservices/createToken.php' +
                          '?id=' + window.localStorage.getItem('fb_id') +
                          '&eventID=' + event.id +
                          '&qrCode=' + event.qrCode +
                          '&date=' + app.prettifyDate(event.date),
                          function () {
          // update the button to not clickable.
          self.el.off('click', '#getFreeStuff', self.getFreeStuff);
          $('#getFreeStuff').css('background-color', '#999');
          $('#getFreeStuff h1').text('Token received.');
          store.updateEventTokenCount(event.id, event.eventDateID, function () {});
          app.showAlert("Check your wallet and scan the token at the event. Have a jol chom!", function () {}, "Token Earned", "Lekka");
        }, function () {
          app.showAlert("Something went wrong when trying to create your token. Please try again!", function () {}, "Error", "Oh no");
        });
    };
    
    openFB.getLoginStatus(function (data) {
      console.log('checking fb status');
      var shouldLogin = false;
//      self.revoke();
      if (data.status === 'connected') {
        var fbId = window.localStorage.getItem('fb_id');
        if (typeof(fbId) == 'undefined' || fbId == null || fbId == 'null') {
          // log in
          shouldLogin = true;
        } else {
          shouldLogin = false;
        }
        console.log('logged in');
      } else {
        console.log('not logged in');
        shouldLogin = true;
      }
      
      if(shouldLogin == true) {
        self.login(function () {
          self.share(successFn);
        });
      } else {
        self.share(successFn);
      }
    });
  };
  
  this.share = function (fn) {
    console.log(event);
    
    //FacebookInAppBrowser.feed({
//      name: event.name,
//      link: 'http://www.themountaingoat.co.za/event.html?id=' + event.eventDateID,
//      picture: event.imageLink,
//      caption: event.venue,
//      description: event.shareContent
//        
//    }, function (response) {
//      if (response) {
//        fn();
//      }
//    });
    
    console.log('about to feed');
    openFB.feedDialog(function (response) {
      if (response) {
        fn();
      }
    }, {
      name: event.name,
      link: 'http://www.themountaingoat.co.za/event.html?id=' + event.eventDateID,
      picture: event.imageLink,
      caption: event.venue,
      description: event.shareContent
        
    });
  };
  
  this.revoke = function () {
    openFB.revokePermissions(
      function () {
        app.showAlert('Permissions revoked successfully', function () {}, 'Permissions', 'Lekka');
      },
      app.errorHandler
    );
  };
  
  this.login = function (successFn) {
    openFB.login(function (response) {
      if (response.status === 'connected') {
        self.getFacebookInfo(successFn);
        //app.showAlert("got access token: " + response.authResponse.token, function () {}, "Facebook login succeeded", "Lekka");
      } else {
        app.showAlert(response.error, function () {}, "Facebook login failed", "Oh no");
      }
    }, {scope: 'email,read_stream,publish_actions,publish_stream'});
  };
  
  this.getFacebookInfo = function (successFn) {
    openFB.api({
      path: '/me',
      success: function (data) {
        console.log(JSON.stringify(data));
//        sessionStorage.setItem('fb_id', data.id);
//        sessionStorage.setItem('fb_first_name', data.first_name);
//        sessionStorage.setItem('fb_last_name', data.last_name);
//        sessionStorage.setItem('email', data.email);
        window.localStorage.setItem('fb_id', data.id);
        window.localStorage.setItem('fb_first_name', data.first_name);
        window.localStorage.setItem('fb_last_name', data.last_name);
        window.localStorage.setItem('email', data.email);
        
        var queryString = '?id=' + data.id + '&fName=' + data.first_name + '&lName=' + data.last_name + '&email=' + data.email;
        
        app.executeJsonCall('http://themountaingoat.co.za/webservices/createUser.php' + queryString, successFn);
      },
      error: app.errorHandler
    });
  };
  
  this.initialize();
};
 
EventView.template = Handlebars.compile($("#event-tpl").html());
EventView.rewardLiTemplate = Handlebars.compile($("#reward-tpl").html());
EventView.noItemTemplate = Handlebars.compile($("#no-item-white-tpl").html());