/*global $, app, console, iScroll, cordova, Handlebars, scanner*/

var WalletView = function (store, dateScreen) {
  "use strict";
  var self = this;
  
  // Application Constructor
  this.initialize = function () {
    this.el = $('<div/>');
    this.el.on('click', '#valid-tokens-div', this.showValidTokens);
    this.el.on('click', '#past-tokens-div', this.showPastTokens);
  };

  this.render = function () {
    this.el.html(WalletView.template(dateScreen));
    this.getTokens();
    return this;
  };

  this.getTokens = function () {
    var fbId = window.localStorage.getItem('fb_id');
    if (typeof(fbId) == 'undefined' || fbId == null) {
      fbId = 0;
    }
    
//    app.executeJsonCall('http://themountaingoat.co.za/webservices/listTokens.php?id=' + sessionStorage.getItem('fb_id'), function (data) {
    app.executeJsonCall('http://themountaingoat.co.za/webservices/listAllTokens.php?id=' + fbId, function (data) {
      console.log(data);
      var splitTokens = self.splitBetweenValidAndPastTokens(data);
      
      if (splitTokens[0].length >= 1) {
        $('#valid-tokens-list').html(WalletView.liTemplate(splitTokens[0]));
      } else {
        $('#valid-tokens-list').html(WalletView.noItemTemplate({ entity: "tokens" }));
      }
      if (splitTokens[1].length >= 1) {
        $('#past-tokens-list').html(WalletView.liTemplate(splitTokens[1]));
      } else {
        $('#past-tokens-list').html(WalletView.noItemTemplate({ entity: "tokens" }));
      }
      //app.setupIScroll(self);
      self.showValidTokens();
    });
  };
  
  this.splitBetweenValidAndPastTokens = function (tokens) {
    var validTokens = [],
      pastTokens = [];
    
    for (var i = 0; i < tokens.length; i++) {
      var token = tokens[i];
      var redeemEndTime = token.redeemEnd.split(':');
      var dateParts = (/\d{4}\-\d{2}\-\d{2}/).exec(token.date)[0].split('-');
      
      if (token.status == 0 &&
          new Date(dateParts[0], dateParts[1]-1, dateParts[2]).setHours(redeemEndTime[0], redeemEndTime[1],0,0) >= new Date().setSeconds(0,0)) {
        validTokens.push(token);
      } else {
        if (token.status == 0) {
          token.status = "2";
        }
        pastTokens.push(token);
      }
    }
    
    return [validTokens, pastTokens];
  }
  
  // Shows the valid tokens list.
  this.showValidTokens = function () {
    self.switchBetweenButtons('valid', 'past');
  };
  
  // Shows the past tokens list.
  this.showPastTokens = function () {
    self.switchBetweenButtons('past', 'valid');
  };

  this.switchBetweenButtons = function (showing, hiding) {
    // The control to turn on
    $('#' + showing + '-tokens-div').addClass('inverted-selected-button');
    $('#' + showing + '-tokens-list').removeClass('hidden');
    $('#' + showing + '-tokens-list-div').css('z-index', '99');

    // The control to turn off
    $('#' + hiding + '-tokens-div').removeClass('inverted-selected-button');
    $('#' + hiding + '-tokens-list').addClass('hidden');
    $('#' + hiding + '-tokens-list-div').css('z-index', '0');
  };

  this.scan = function (tokenId, qrCode, venueName, redeemStart, redeemEnd, date) {
    console.log('scanning');
    
    var redeemStartTime = redeemStart.split(':');
    var dateParts = (/\d{4}\-\d{2}\-\d{2}/).exec(date)[0].split('-');
    var currentDate = new Date().setSeconds(0,0);
    if (currentDate <= new Date(dateParts[0], dateParts[1]-1, dateParts[2]).setHours(redeemStartTime[0], redeemStartTime[1],0,0)) {
      app.showAlert("Too soon to scan token, please check Redeem Period.", function () {}, "Too Early", "Oh no");
      return;
    }
    
    var scanner = cordova.require("cordova/plugin/BarcodeScanner");

    scanner.scan(function (result) {
      if (result.cancelled == true) { return; }

      console.log(result);

      if (result.format == "QR_CODE") {
        if (result.text == qrCode) {
          app.executeJsonCall('http://themountaingoat.co.za/webservices/redeemToken.php?id=' + tokenId, function (data) {
            app.showAlert("Your '" + app.unPrettifyText(venueName) + "' token has been successfully redeemed. Please show this message to a venue employee.", function () {}, "Token Redeemed", "Lekka");
            self.getTokens();
          });
        } else {
          app.showAlert("The QR Code does not match the token. Select a different token and try again.", function () {}, 'Incorrect QR Code', "Oh no");
        }
      } else {
        app.showAlert("Barcode was not a QR Code. Please try again by scanning a QR Code", function () {}, 'Wrong barcode type', "Oh no");
      }
    }, function (error) {
      console.log("Scanning failed: ", error);
      app.errorHandler("Scanning failed : " + error);
    });
  };

  this.initialize();
};

WalletView.template = Handlebars.compile($("#wallet-tpl").html());
WalletView.liTemplate = Handlebars.compile($("#token-li-tpl").html());
WalletView.noItemTemplate = Handlebars.compile($("#no-item-black-tpl").html());