var MemoryStore = function (successCallback, errorCallback) {
  "use strict";

  // Used to simulate async calls. This is done to provide a consistent interface with stores (like WebSqlStore)
  // that use async data access APIs
  var callLater = function (callback, data) {
    if (callback) {
      setTimeout(function () {
        callback(data);
      });
    }
  };
  
  this.findByName = function (searchKey, callback) {
    var employees = this.employees.filter(function (element) {
      var fullName = element.firstName + " " + element.lastName;
      return fullName.toLowerCase().indexOf(searchKey.toLowerCase()) > -1;
    });
    callLater(callback, employees);
  };
  
  this.getDateScreen = function (callback) {
    var currentDateScreen = "Today";
    if (this.currentDateScreen) {
      var currentDateScreen = this.currentDateScreen;
    }
    callLater(callback, currentDateScreen);
  };
  
  this.setDateScreen = function (dateScreen) {
    this.currentDateScreen = dateScreen;
  };

  this.findById = function (id, callback) {
    var employees = this.employees;
    var employee = null;
    var l = employees.length;
    for (var i=0; i < l; i++) {
      if (employees[i].id === id) {
        employee = employees[i];
        break;
      }
    }
    callLater(callback, employee);
  };

  this.getEventListForToday = function(callback) {
    var events = this.eventsTodayListItems;
    callLater(callback, events);
  };

  this.storeEventsListForToday = function(events) {
    this.eventsTodayListItems = this.eventsTodayListItems.concat(events);
  };

  this.getEventListForTomorrow = function(callback) {
    var events = this.eventsTomorrowListItems;
    callLater(callback, events);
  };

  this.storeEventsListForTomorrow = function(events) {
    this.eventsTomorrowListItems = this.eventsTomorrowListItems.concat(events);
  };

  this.getEventListForTheNextDay = function(callback) {
    var events = this.eventsTheNextDayListItems;
    callLater(callback, events);
  };

  this.storeEventsListForTheNextDay = function(events) {
    this.eventsTheNextDayListItems = this.eventsTheNextDayListItems.concat(events);
  };
  
  this.updateEventTokenCount = function(id, eventDateID, callback) {
    var events = this.eventInfos;
    var event = null;
    var l = events.length;
    for (var i=0; i < l; i++) {
      if (events[i].id == id && events[i].eventDateID == eventDateID) {
        event = events[i];
        break;
      }
    }
    
    event.count = event.count + 1;
    callLater(callback, event);
  };

  this.findEventById = function(id, date, callback) {
    var events = this.eventInfos;
    var event = null;
    var l = events.length;
    for (var i=0; i < l; i++) {
      if (events[i].id == id && events[i].date == date) {
        event = events[i];
        break;
      }
    }
    callLater(callback, event);
  };

  this.findVenueById = function(id, callback) {
    var venues = this.venueInfos;
    var venue = null;
    var l = venues.length;
    for (var i=0; i < l; i++) {
      if (venues[i].venueID == id) {
        venue = venues[i];
        break;
      }
    }
    callLater(callback, venue);
  };

  this.getSpecialListByEventId = function (eventDateID, callback) {
    var specials = this.specialListItems;
    var filteredSpecials = [];
    var l = specials.length;
    for (var i=0; i < l; i++) {
      if (specials[i].eventDateID == eventDateID) {
        filteredSpecials.push(specials[i]);
      }
    }
    callLater(callback, filteredSpecials);
  };

  this.getRewardListByEventId = function(eventDateID, callback) {
    var rewards = this.rewardListItems;
    var filteredRewards = [];
    var l = rewards.length;
    for (var i=0; i < l; i++) {
      if (rewards[i].eventDateID == eventDateID) {
        filteredRewards.push(rewards[i]);
      }
    }
    callLater(callback, filteredRewards);
  };

  this.storeSpecialList = function (specials, eventDateID) {
    specials.forEach(function (part, index, arr){ arr[index].eventDateID = eventDateID;});
    this.specialListItems = this.specialListItems.concat(specials);
  };

  this.storeRewardList = function (rewards, eventDateID) {
    rewards.forEach(function (part, index, arr){ arr[index].eventDateID = eventDateID;});
    this.rewardListItems = this.rewardListItems.concat(rewards);
  };
  
  this.storeVenue = function (venue) {
    this.venueInfos.push(venue);
  };
  
  this.storeEvent = function (event) {
    this.eventInfos.push(event);
  };

  this.getTokenListByUserId = function(id, callback) {
    var tokens = this.tokenListItems;
    var filteredTokens = [];
    var l = tokens.length;
    for (var i=0; i < l; i++) {
      if (specials[i].id == id) {
        filteredTokens.push(tokens[i]);
      }
    }
    callLater(callback, filteredTokens);
  };
  
  this.clearStore = function () {
    this.eventsTodayListItems = [];
    this.eventsTomorrowListItems = [];
    this.eventsTheNextDayListItems = [];
    this.eventInfos = [];
    this.venueInfos = [];
    this.specialListItems = [];
    this.rewardListItems = [];
  };
  

  this.eventsTodayListItems = [];
  this.eventsTomorrowListItems = [];
  this.eventsTheNextDayListItems = [];/*[{"id":"1","name":"Active Sushi","venue":"Active Sushi","imageLink":"http:\/\/197.242.147.119\/images\/activesushi\/eventImage.png","totalViews":"302","totalShares":"66"},{"id":"2","name":"Side Show Friday Nights","venue":"The Side Show","imageLink":"http:\/\/197.242.147.119\/images\/thesideshow\/eventImage.jpg","totalViews":"96","totalShares":"26"},{"id":"3","name":"Half Price Pizza","venue":"Rafiki's","imageLink":"http:\/\/197.242.147.119\/images\/rafikis\/monday.png","totalViews":"73","totalShares":"11"}];*/
  this.eventInfos = [];/*[{"id":"1","name":"Active Sushi","date":"2014-09-17","description":"Lorem ipsum dolor sit amet, consectetuer adipiscing elit, sed diam nonummy nibh euismod tincidunt ut laoreet dolore magna aliquam erat volutpat. Ut wisi enim ad minim veniam, quis nostrud exerci tation ullamcorper suscipit lobortis nisl ut aliquip ex","imageLink":"http:\/\/197.242.147.119\/images\/activesushi\/eventImage.png","coverCharge":"N\/A","ageRestriction":"N\/A","startTime":"11:30:00","endTime":"22:30:00","shareContent":"Going to Active Sushi today for their super cheap specials. Starting at 16 pieces for R50 or the All you can eat for R119! ","venueID":"1","qrCode":"ACt1v3","count":"0"},{"id":"2","name":"Side Show Friday Nights","date":"2014-09-18","description":"Lorem ipsum dolor sit amet, consectetuer adipiscing elit, sed diam nonummy nibh euismod tincidunt ut laoreet dolore magna aliquam erat volutpat. Ut wisi enim ad minim veniam, quis nostrud exerci tation ullamcorper suscipit lobortis nisl ut aliquip ex","imageLink":"http:\/\/197.242.147.119\/images\/thesideshow\/eventImage.jpg","coverCharge":"R100","ageRestriction":"18+","startTime":"21:00:00","endTime":"02:00:00","shareContent":"xcxcvxc","venueID":"2","qrCode":"SiDe5h0w","count":"0"},{"id":"3","name":"Half Price Pizza","date":"2014-09-18","description":"Monday's at Rafikis are not only filled with laughter, a great vibe and tons of awesome people but also your pizza is half price. All day. All night.","imageLink":"http:\/\/197.242.147.119\/images\/rafikis\/monday.png","coverCharge":"N\/A","ageRestriction":"18+","startTime":"11:00:00","endTime":"22:00:00","shareContent":"Pulling into Rafikis for their all day\/night half price pizza. Pull in chaps!","venueID":"3","qrCode":"RaFiKiS","count":"0"}];*/
  this.venueInfos = [];/*[{"id":"1","name":"Active Sushi","description":"Lorem ipsum dolor sit amet, consectetuer adipiscing elit, sed diam nonummy nibh euismod tincidunt ut laoreet dolore magna aliquam erat volut","address":"Corner of Bree & Mechau Street.","gpsLat":"-33.917257","gpsLon":"18.422269","website":"http:\/\/www.activesushi.com","imageLink":"http:\/\/197.242.147.119\/images\/activesushi\/eventImage.png"}];*/
  this.specialListItems = [];/*[{"id":"1","name":"Active 1","description":"6 Maki, 4 California Rolls, 4 Fashion Sandwiches, 2 Nigiri","price":"50"},{"id":"2","name":"Active 2","description":"6 Maki, 4 California Rolls, 4 Fashion Sandwiches, 2 Nigiri, 2 Roses","price":"60"},{"id":"3","name":"Active 3","description":"6 Maki, 4 California Rolls, 4 Fashion Sandwiches, 2 Nigiri, 2 Roses, 2 Sashimi","price":"70"},{"id":"4","name":"All You Can Eat","description":"Eat as much as you like. Includes sashimi, nigiri and roses.","price":"119"}];*/
  this.rewardListItems = [];/*[{"id":"1","name":"Discount","description":"Get 10% off your individual order","limit":"20","redeemStart":"11:30:00","redeemEnd":"20:00:00"}];*/
  this.tokenListItems = [];/*[{"id":"19","userID":"10202631442006464","qrCode":"ACt1v3","date":"2014-09-19 00:00:00","status":"1","name":"Active Sushi","venueName":"Active Sushi"},{"id":"20","userID":"10202631442006464","qrCode":"ACt1v3","date":"2014-09-19 00:00:00","status":"0","name":"Active Sushi","venueName":"Active Sushi"},{"id":"21","userID":"10202631442006464","qrCode":"ACt1v3","date":"2014-09-20 00:00:00","status":"0","name":"Active Sushi","venueName":"Active Sushi"}]*/

/*  this.employees = [
          {"id": 1, "firstName": "Ryan", "lastName": "Howard", "title":"Vice President, North East", "managerId": 0, "city":"New York, NY", "cellPhone":"212-999-8888", "officePhone":"212-999-8887", "email":"ryan@dundermifflin.com"},
          {"id": 2, "firstName": "Michael", "lastName": "Scott", "title":"Regional Manager", "managerId": 1, "city":"Scranton, PA", "cellPhone":"570-865-2536", "officePhone":"570-123-4567", "email":"michael@dundermifflin.com"},
          {"id": 3, "firstName": "Dwight", "lastName": "Schrute", "title":"Assistant Regional Manager", "managerId": 2, "city":"Scranton, PA", "cellPhone":"570-865-1158", "officePhone":"570-843-8963", "email":"dwight@dundermifflin.com"},
          {"id": 4, "firstName": "Jim", "lastName": "Halpert", "title":"Assistant Regional Manager", "managerId": 2, "city":"Scranton, PA", "cellPhone":"570-865-8989", "officePhone":"570-968-5741", "email":"dwight@dundermifflin.com"},
          {"id": 5, "firstName": "Pamela", "lastName": "Beesly", "title":"Receptionist", "managerId": 2, "city":"Scranton, PA", "cellPhone":"570-999-5555", "officePhone":"570-999-7474", "email":"pam@dundermifflin.com"},
          {"id": 6, "firstName": "Angela", "lastName": "Martin", "title":"Senior Accountant", "managerId": 2, "city":"Scranton, PA", "cellPhone":"570-555-9696", "officePhone":"570-999-3232", "email":"angela@dundermifflin.com"},
          {"id": 7, "firstName": "Kevin", "lastName": "Malone", "title":"Accountant", "managerId": 6, "city":"Scranton, PA", "cellPhone":"570-777-9696", "officePhone":"570-111-2525", "email":"kmalone@dundermifflin.com"},
          {"id": 8, "firstName": "Oscar", "lastName": "Martinez", "title":"Accountant", "managerId": 6, "city":"Scranton, PA", "cellPhone":"570-321-9999", "officePhone":"570-585-3333", "email":"oscar@dundermifflin.com"},
          {"id": 9, "firstName": "Creed", "lastName": "Bratton", "title":"Quality Assurance", "managerId": 2, "city":"Scranton, PA", "cellPhone":"570-222-6666", "officePhone":"570-333-8585", "email":"creed@dundermifflin.com"},
          {"id": 10, "firstName": "Andy", "lastName": "Bernard", "title":"Sales Director", "managerId": 4, "city":"Scranton, PA", "cellPhone":"570-555-0000", "officePhone":"570-646-9999", "email":"andy@dundermifflin.com"},
          {"id": 11, "firstName": "Phyllis", "lastName": "Lapin", "title":"Sales Representative", "managerId": 10, "city":"Scranton, PA", "cellPhone":"570-241-8585", "officePhone":"570-632-1919", "email":"phyllis@dundermifflin.com"},
          {"id": 12, "firstName": "Stanley", "lastName": "Hudson", "title":"Sales Representative", "managerId": 10, "city":"Scranton, PA", "cellPhone":"570-700-6464", "officePhone":"570-787-9393", "email":"shudson@dundermifflin.com"},
          {"id": 13, "firstName": "Meredith", "lastName": "Palmer", "title":"Supplier Relations", "managerId": 2, "city":"Scranton, PA", "cellPhone":"570-588-6567", "officePhone":"570-981-6167", "email":"meredith@dundermifflin.com"},
          {"id": 14, "firstName": "Kelly", "lastName": "Kapoor", "title":"Customer Service Rep.", "managerId": 2, "city":"Scranton, PA", "cellPhone":"570-123-9654", "officePhone":"570-125-3666", "email":"kelly@dundermifflin.com"},
          {"id": 15, "firstName": "Toby", "lastName": "Flenderson", "title":"Human Resources", "managerId": 1, "city":"Scranton, PA", "cellPhone":"570-485-8554", "officePhone":"570-699-5577", "email":"toby@dundermifflin.com"}
      ];*/

  this.homePageSetting = { Day : 'Today', FollowingDay: 'Tomorrow' };

  callLater(successCallback);
}