/*global console, alert */
var WebSqlStore = function (successCallback, errorCallback) {
  'use strict';
  
  this.initializeDatabase = function (successCallback, errorCallback) {
    var self = this;
    this.db = window.openDatabase("TMGDB", "1.0", "TMG DB", 200000);
    this.db.transaction(
      function (tx) {
        self.createUserTable(tx);
        self.createPostTable(tx);
      },
      function (error) {
        console.log('Transaction error: ' + error);
        if (errorCallback) {
          errorCallback();
        }
      },
      function () {
        console.log('Transaction success');
        if (successCallback) {
          successCallback();
        }
      }
    );
  };

  this.createUserTable = function (tx) {
    var sql = "CREATE TABLE IF NOT EXISTS fbUser ( " +
        "token VARCHAR(256))";
    tx.executeSql(sql, null,
      function () {
        console.log('Create table success');
      },
      function (tx, error) {
        alert('Create table error: ' + error.message);
      });
  };

  this.createPostTable = function (tx) {
    var sql = "CREATE TABLE IF NOT EXISTS fbPost ( " +
        "postId VARCHAR(256))";
    tx.executeSql(sql, null,
      function () {
        console.log('Create table success');
      },
      function (tx, error) {
        alert('Create table error: ' + error.message);
      });
  };
  
  this.setFbToken = function (token, successCallback, errorCallback) {
    var self = this;
    this.db.transaction(
      function (tx) {
        self.saveFBToken(tx, token);
      },
      function (error) {
        console.log('Transaction error: ' + error);
        if (errorCallback) {
          errorCallback();
        }
      },
      function () {
        console.log('Transaction success');
        if (successCallback) {
          successCallback();
        }
      }
    );
  };
  
  this.saveFBToken = function (tx, token) {
    var sql = "INSERT OR REPLACE INTO fbUser " +
        "(token) " +
        "VALUES (?)";
    tx.executeSql(sql, [token],
      function () {
        console.log('INSERT success');
      },
      function (tx, error) {
        alert('INSERT error: ' + error.message);
      });
  };

  this.getFbToken = function (callback) {
    var token = null;
    this.db.transaction(
      function (tx) {
        var sql = "SELECT fb.token " +
            "FROM fbUser fb";
        tx.executeSql(sql, [], function (tx, results) {
          token = results.rows.length === 1 ? results.rows.item(0) : null;
          if (callback) {
            callback(results.rows.length === 1 ? results.rows.item(0) : null);
          }
        });
      },
      function (error) {
        alert("Transaction Error: " + error.message);
      }
    );
    return token;
  };

  this.removeFBToken = function (callback) {
    this.db.transaction(
      function (tx) {
        var sql = "DELETE fb.token " +
            "FROM fbUser fb";
        tx.executeSql(sql, [], function (tx, results) {
          if (callback) {
            callback(results.rows.length === 1 ? results.rows.item(0) : null);
          }
        });
      },
      function (error) {
        alert("Transaction Error: " + error.message);
      }
    );
  };
  
  this.setFbPostId = function (postId, successCallback, errorCallback) {
    var self = this;
    this.db.transaction(
      function (tx) {
        self.saveFbPostId(tx, postId);
      },
      function (error) {
        console.log('Transaction error: ' + error);
        if (errorCallback) {
          errorCallback();
        }
      },
      function () {
        console.log('Transaction success');
        if (successCallback) {
          successCallback();
        }
      }
    );
  };
  
  this.saveFbPostId = function (tx, postId) {
    var sql = "INSERT OR REPLACE INTO fbPost " +
        "(postId) " +
        "VALUES (?)";
    tx.executeSql(sql, [postId],
      function () {
        console.log('INSERT success');
      },
      function (tx, error) {
        alert('INSERT error: ' + error.message);
      });
  };
  
  this.initializeDatabase(successCallback, errorCallback);
};
